@@ -136,6 +136,9 @@ average_treatment_effect <- function(forest,
     if (cluster.se) {
       stop("TMLE has not yet been implemented with clustered observations.")
     }
+    if (!is.null(forest$sample.weights)) {
+      stop("TMLE has not yet been implemented with sample weighted observations.")
+    }
   }
 
   clusters <- if (cluster.se) {
@@ -200,9 +203,10 @@ average_treatment_effect <- function(forest,
     .sigma2.hat <- function(DR.scores, tau.hat) {
       correction.clust <- Matrix::sparse.model.matrix(~ factor(subset.clusters) + 0, transpose = TRUE) %*%
         (sweep(as.matrix(DR.scores), 2, tau.hat, "-") * subset.weights)
+      n.adj <- sum(rowsum(subset.weights, subset.clusters) != 0) # effective number of samples(clusters) is all with > 0 weight.
 
       Matrix::colSums(correction.clust^2) / sum(subset.weights)^2 *
-        nrow(correction.clust) / (nrow(correction.clust) - 1)
+        n.adj / (n.adj - 1)
     }
 
     if (any(c("causal_forest", "instrumental_forest", "multi_arm_causal_forest", "causal_survival_forest")
@@ -258,6 +262,7 @@ average_treatment_effect <- function(forest,
   subset.Y.hat.1 <- subset.Y.hat + (1 - subset.W.hat) * tau.hat.pointwise
 
   if (target.sample == "overlap") {
+    validate_sandwich(subset.weights)
 
     # Address the overlap case separately, as this is a very different estimation problem.
     # The method argument (AIPW vs TMLE) is ignored in this case, as both methods are effectively
@@ -349,10 +354,12 @@ average_treatment_effect <- function(forest,
         ~ factor(subset.clusters) + 0,
         transpose = TRUE
       ) %*% (dr.correction.all * subset.weights)
+      n.adj <- sum(rowsum(subset.weights, subset.clusters) != 0) # effective number of clusters is all with > 0 weight.
       sigma2.hat <- sum(correction.clust^2) / sum(subset.weights)^2 *
-        length(correction.clust) / (length(correction.clust) - 1)
+        n.adj / (n.adj - 1)
     } else {
-      sigma2.hat <- mean(dr.correction.all^2) / (length(dr.correction.all) - 1)
+      sigma2.hat <- sum(subset.weights^2 * dr.correction.all^2 / sum(subset.weights)^2) *
+        length(subset.weights[subset.weights != 0]) / (length(subset.weights[subset.weights != 0]) - 1)
     }
   } else if (method == "TMLE") {
     if (target.sample == "all") {
