@@ -15,22 +15,24 @@
   along with grf. If not, see <http://www.gnu.org/licenses/>.
  #-------------------------------------------------------------------------------*/
 
-#ifndef GRF_LEAFNODEFINDER_H
-#define GRF_LEAFNODEFINDER_H
+#ifndef GRF_TREETRAVERSER_H
+#define GRF_TREETRAVERSER_H
 
 #include "forest/Forest.h"
 
+namespace grf {
+
 class TreeTraverser {
 public:
   TreeTraverser(uint num_threads);
 
   std::vector<std::vector<size_t>> get_leaf_nodes(
       const Forest& forest,
-      Data* data,
+      const Data& data,
       bool oob_prediction) const;
 
   std::vector<std::vector<bool>> get_valid_trees_by_sample(const Forest& forest,
-                                                           Data* data,
+                                                           const Data& data,
                                                            bool oob_prediction) const;
 
 private:
@@ -38,15 +40,16 @@ private:
       size_t start,
       size_t num_trees,
       const Forest& forest,
-      Data* data,
+      const Data& data,
       bool oob_prediction) const;
 
   std::vector<bool> get_valid_samples(size_t num_samples,
-                                      std::shared_ptr<Tree> tree,
+                                      const std::unique_ptr<Tree>& tree,
                                       bool oob_prediction) const;
 
   uint num_threads;
 };
 
+} // namespace grf
 
-#endif //GRF_LEAFNODEFINDER_H
+#endif //GRF_TREETRAVERSER_H
