@@ -39,15 +39,8 @@ void RandomSampler::sample_clusters(size_t num_rows,
 void RandomSampler::sample(size_t num_samples,
                            double sample_fraction,
                            std::vector<size_t>& samples) {
-  size_t num_samples_inbag = (size_t) num_samples * sample_fraction;
-  if (options.get_sample_weights().empty()) {
-    shuffle_and_split(samples, num_samples, num_samples_inbag);
-  } else {
-    draw_weighted(samples,
-                  num_samples - 1,
-                  num_samples_inbag,
-                  options.get_sample_weights());
-  }
+  size_t num_samples_inbag = static_cast<size_t>(num_samples * sample_fraction);
+  shuffle_and_split(samples, num_samples, num_samples_inbag);
 }
 
 void RandomSampler::subsample(const std::vector<size_t>& samples,
@@ -194,36 +187,15 @@ void RandomSampler::draw_fisher_yates(std::vector<size_t>& result,
   // Draw without replacement using Fisher Yates algorithm
   nonstd::uniform_real_distribution<double> distribution(0.0, 1.0);
   for (size_t i = 0; i < num_samples; ++i) {
-    size_t j = i + distribution(random_number_generator) * (max - skip.size() - i);
+    size_t j = static_cast<size_t>(i + distribution(random_number_generator) * (max - skip.size() - i));
     std::swap(result[i], result[j]);
   }
 
   result.resize(num_samples);
 }
 
-void RandomSampler::draw_weighted(std::vector<size_t>& result,
-                                  size_t max,
-                                  size_t num_samples,
-                                  const std::vector<double>& weights) {
-  result.resize(num_samples);
-
-  // Set all to not selected
-  std::vector<bool> temp;
-  temp.resize(max + 1, false);
-
-  nonstd::discrete_distribution<> weighted_dist(weights.begin(), weights.end());
-  for (size_t i = 0; i < num_samples; ++i) {
-    size_t draw;
-    do {
-      draw = weighted_dist(random_number_generator);
-    } while (temp[draw]);
-    temp[draw] = true;
-    result[i] = draw;
-  }
-}
-
 size_t RandomSampler::sample_poisson(size_t mean) {
-  nonstd::poisson_distribution<size_t> distribution(mean);
+  nonstd::poisson_distribution<size_t> distribution(static_cast<double>(mean));
   return distribution(random_number_generator);
 }
 
